
#import "CalendarController.h"

int days_in_month[]={0,31,28,31,30,31,30,31,31,30,31,30,31};
int determinedaycode(int year)
{
	int daycode;
	int d1, d2, d3;
	
	d1 = (year - 1.)/ 4.0;
	d2 = (year - 1.)/ 100.;
	d3 = (year - 1.)/ 400.;
	daycode = (year + d1 - d2 + d3) %7;
	return daycode;
}

int determineleapyear(int year)
{
	return (year% 4 == FALSE && year%100 != FALSE || year%400 == FALSE) ;
}

int leapday(int month,int leapyear)  { return (month==2 && leapyear) ? 1 : 0 ; }
int daysInMonth(int year, int month) { return days_in_month[month]+leapday(month,determineleapyear(year)) ; }

int getStartOfMonth(int year, int M)
{
	int daycode = determinedaycode(year) ;
	for ( int month = 1; month < M; month++ )
	{
		// Set position for next month
		daycode = ( daycode + daysInMonth(year,month) ) % 7;
	}
	return daycode ;
}


@implementation CalendarController

static NSString* sMonths[] = 
{	@"January"		, @"February"	, @"March"
,	@"April"		, @"May"		, @"June"
,	@"July"			, @"August"		, @"September"
,	@"October"		, @"November"	, @"December"
};

- (void) awakeFromNib
{
	baseYear = 1980 ;
	[controlYear removeAllItems];
	for ( int y = 0 ; y <= 50 ; y++ ) 
	{
		NSString* s = [NSString stringWithFormat:@"%d",y+baseYear];
		[controlYear addItemWithTitle:s];
	}
	
	baseMonth=1;
	[controlMonth removeAllItems];
	for ( int m = 0 ; m < 12 ; m++ ) 
	{
		[controlMonth addItemWithTitle:sMonths[m]];
	}
	
	NSDate* now = [NSDate dateWithTimeIntervalSinceNow:0];

	NSInteger nowYear  = [[[NSCalendar currentCalendar]components:NSYearCalendarUnit fromDate:now]year] - baseYear;
	[controlYear selectItemAtIndex:nowYear];
	
	NSInteger nowMonth  = [[[NSCalendar currentCalendar]components:NSMonthCalendarUnit fromDate:now]month]-baseMonth ;
	[controlMonth selectItemAtIndex:nowMonth];
	
	[self update:nil];
	[self createMatrix];
	[self createCalendar];
	

}


#define ROWS 6
#define COLUMNS 7

-(void)createMatrix {
	// Creating the Prototype
	protoCell = [[NSButtonCell alloc] init];
	[protoCell setButtonType:NSToggleButton];
	[protoCell setImage:[NSImage imageNamed:@"unused.png"]];
	[protoCell setBordered:NO];
	[protoCell setTitle:@""];
	
	
	// Creating the Matrix
	NSRect matrixFrame = [matrixView frame];
	matrixDays = [[NSMatrix alloc] initWithFrame:matrixFrame
											mode:NSRadioModeMatrix
									   prototype:protoCell
									numberOfRows:ROWS numberOfColumns:COLUMNS ];
	
	
	[matrixDays setAutosizesCells:YES];
	
	[matrixDays setTarget:self];
	[matrixBox addSubview:matrixDays];
	[matrixView removeFromSuperview];
}
	
-(void)createCalendar {
	// Creating the Current Date
	NSDate *current = [NSDate date];
	NSInteger currentDay = [[[NSCalendar currentCalendar]components:NSDayCalendarUnit fromDate:current]day];
	NSInteger currentMonth  = [[[NSCalendar currentCalendar]components:NSMonthCalendarUnit fromDate:current]month];
	NSInteger currentYear  = [[[NSCalendar currentCalendar]components:NSYearCalendarUnit fromDate:current]year];
		
	// Label the Cells to Correct Date
	int row, column;
		
	for (row=0; row<ROWS; row++)
		for (column=0; column<COLUMNS; column++){
			matrixCell = [matrixDays cellAtRow:row column:column];
				
			int result = column + row*7 - firstDayOfMonth ;
			if ( result > daysInMonth(year,month) ) result = 0 ;
				
			result > 0 ? [matrixCell setTitle:[NSString stringWithFormat:@"%i",result]]:[matrixCell setTitle:@""];
				
			// Used Image if current day
			if (currentDay == result && currentMonth == month && currentYear == year) {
				[matrixCell setImage:[NSImage imageNamed:@"used.png"]];
			}else {
				[matrixCell setImage:[NSImage imageNamed:@"unused.png"]];
			}
				
				
		}
		
}
	
	
- (void) update:(id)sender {
	//	NSLog(@"update");
	year = [controlYear indexOfSelectedItem]+baseYear;
	month = [controlMonth indexOfSelectedItem]+baseMonth ;
	firstDayOfMonth = getStartOfMonth(year,month)-1;
		
	[self createCalendar];
}
	


- (void) fixem
{
	if ( year < baseYear ) year = baseYear ;
	[controlMonth selectItemAtIndex:month];
	[controlYear selectItemAtIndex:(year - baseYear)];
	[self update:nil];
}

- (IBAction) forward:(id)sender
{
	if ( month > 11 ) {
		month = 0 ;
		year++ ;
	}
	[self fixem];
}

- (IBAction) backward:(id)sender
{
	month--;
	if ( !month ) {
		month = 12 ;
		year-- ;
	}
	month-- ;
	[self fixem];
}

-(void)dealloc {
	[super dealloc];
	[super release];
}


@end
